(ns lu-rescaling.core
  (:require [lu-rescaling.rescaling :as rescaling]
            [utilities.writer :as writer]
            [utilities.cmd :as cmd])
  (:gen-class))


;;; private methods

;; Defines description of command line arguments
(def ^{:private true} cli-args
  {:required
   [["map-folder" "Path to the folder with original GIS-based maps"]
    ["region-path" "Path to the file with administrative units"]]})


;;; public methods

(defn -main
  "Upscales the GIS-based variables to the level of administrative
   division by averaging the values on the GIS lattice.

   Saves results into a csv file
   in the root execution directory.

   Arguments: map-folder
              region-path

   Original GIS-based maps are read from the .asc files by the [map-folder] address.
   Reference map of regional administrative division is read
   from an .asc file by the [region-path] address.

   Each position on the regional map is assigned to an identifier of the
   administrative unit, to which this location belongs.

   Upscaling (averaging): values are averaged through administrative unit.
       output: lu-rescaling/sample.csv

   Saved csv table contains tuples of administrative unit id and
   averages in every map provided.
   The first row contains file names of the source maps.
   The first column contains identifiers of administrative units.

   ## Usage

        $ lein run \"GIS raster maps\" sections.asc

  "
  [& args]
  (cmd/terminal {:short-desc "Standalone application for rescaling .asc data."
                 :args args
                 :args-desc cli-args
                 :execute
                 (fn[arguments]
                   (let [[map-folder region-path] arguments]
                     (->> (rescaling/aggregate map-folder
                                               region-path)
                          (writer/csv-file "lu-rescaling"
                                           "sample.csv")
                          (#(println (str "Executed. Results saved to: \""
                                          %
                                          "\""))))))}))
