(ns lu-rescaling.provider
  (:require [clojure.string :as string]
            [utilities.format :as format]))


;;; private methods

(defn- to-row
  "Converts values to a list of formatted strings.
      coll: List of values.
   returns: A list of formatted values."
  [coll]
  (->> (map #(cond
              (zero? %) 0
              (or (ratio? %)
                  (float? %)) (format/double-to-str (double %) 6)
              :else %)
            coll)
       (map str)
       vec))


;;; public methods

(defn to-csv
  "Converts data objects to csv format.
      values: A hashmap of column names and values.
      first-id: A name of the first column.
   returns: A list of csv formatted rows.

  ## Usage

      (require '[lu-rescaling.provider :refer :all])

      (def values {\"id1\" '(1.0 2.0 3.0)
                   \"id2\" '(4 5.0 6.0)
                   \"primary_id\" '(1.0 1.0 1.0)
                   \"id3\" '(7.0 8.0 9.0)})
      (def first-id \"primary_id\")

      (to-csv values first-id)
      => '([\"primary_id\" \"id1\" \"id2\" \"id3\"]
           [\"1.000000\" \"1.000000\" \"4\" \"7.000000\"]
           [\"1.000000\" \"2.000000\" \"5.000000\" \"8.000000\"]
           [\"1.000000\" \"3.000000\" \"6.000000\" \"9.000000\"])
  "
  [values first-id]
  (->> (keys values)
       (filter #(not (= first-id %)))
       sort
       (#(conj % first-id))
       ((juxt vec
              (fn[ids]
                (->> (map #(get values %)
                          ids)
                     (apply map
                            (comp to-row
                                  vector))))))
       (apply cons)))
