(ns lu-rescaling.rescaling
 (:require [lu-rescaling.provider :as provider]
           [lu-rescaling.reader :as reader]
           [lu-rescaling.stat :as stat]))


;;; public methods

(defn aggregate
  "Averages the values of the GIS-based variables in the
   administrative units of the region.
      map-folder: A path to the folder with original GIS-based maps (in .asc format).
      region-path: A path to an .asc file with regional administrative division.
   returns: A csv file content with administrative units and corresponding average values.

  ## Usage

      (require '[lu-rescaling.rescaling :refer :all])

      (aggregate \"GIS raster maps\" sections.asc)
  "
  [map-folder region-path]
  (let [[region-id region]
        (->> ((juxt keys
                    vals)
              (reader/parse-region-map region-path))
             (map first))]
    (->> (reader/parse-maps map-folder)
         (stat/average-groupped region)
         (#(assoc % region-id (keys region)))
         (#(provider/to-csv % region-id)))))
