(ns lu-rescaling.stat
 (:require [lu-rescaling.map-operations :as operations]))


;;; public methods

(defn average-groupped
  "Averages values in the subsets defined by the reference map.
      ref-map: A hashmap of reference values positions.
      target-maps: Hashmaps of values positions.
   returns: A hashmap of id and list of subset averages.

  ## Usage

      (require '[lu-rescaling.stat :refer :all])

      (def ref-map {1 {0 [0 1 3] 1 [2 3]}
                    2 {0 [2] 1 [1]}})
      (def target-maps {\"id1.asc\" {0 {0 [0]} 1 {1 [1 2]} 2 {0 [1 2] 1 [3]} 3 {0 [3]}}
                        \"id2.asc\" {1 {0 [0 1 2 3] 1 [1 2 3]}}})

      (average-groupped ref-map target-maps)
      => {\"id1.asc\" '(8/5 3/2)
          \"id2.asc\" '(1 1)}"
  [ref-map target-maps]
  (->> (operations/merge-into-cross-map ref-map
                                        target-maps)
       vals
       (map (fn[v]
              (->> (vals v)
                   (map #(-> (apply + %)
                             (/ (count %))
                             list))
                   (zipmap (keys v)))))
       (apply merge-with concat)))
