(ns data-cleaning.sample)


;;; public methods

(defn get-neighbourhood-data
  "Returns frequency of different types of Moore neighbourhood.
      neighbour-matrices: Matrices of neighbour (and self) land use type distribution.
   returns: A hash map with Moore neighbourhood type as key and
            and it's frequency.

  ## Usage

      (require '[data-cleaning.sample :refer :all])

      (get-neighbourhood-data [[1 1 1 1 1 1 1 1 1]
                               [0 0 0 0 1 0 0 0 0]
                               [0 0 0 0 1 0 0 0 0]
                               [0 0 0 0 0 0 0 0 0]
                               [1 nil 0 0 1 0 0 0 1]
                               [0 0 0 0 0 1 0 0 0]
                               [0 0 0 0 1 1 0 0 0]
                               [1 1 1 1 1 1 1 1 1]
                               [0 0 0 0 0 0 0 0 0]])
      => {[1 0 0 0 1 0 0 1 0] 2
          [1 0 0 0 0 0 0 1 0] 4
          [1 1 1 0 1 0 1 1 0] 1
          [1 0 0 0 0 1 1 1 0] 1}
  "
  [neighbour-matrices]
  (let [neighbourhoods
        (->> (apply mapv vector neighbour-matrices)
             (filter #(not (some nil? %))))]
    (->> (distinct neighbourhoods)
         (map #(->> neighbourhoods
                    (filter (fn[combination]
                              (= % combination)))
                    count
                    (hash-map %)))
         (into {}))))
