(ns data-cleaning.filter-test
  (:require [clojure.test :refer :all]
            [data-cleaning.filter :refer :all]))

;;; tests

(deftest mask-na-cells-and-their-neighbourhood
  (testing "Filter should mask cells,
    whose value or their neighbour value are not defined."
    (
     ; Arrange
     let [map-matrix ["1" "2" "2" "3" "4" "1"
                      "3" "na" "3" "5" "6" "7"
                      "2" "8" "1" "2" "3" "1"
                      "1" "3" "2" "8" "1" "1"
                      "5" "4" "7" "na" "na" "1"
                      "6" "5" "1" "1" "1" "1"]
          na-values {1 [1] 4 [3 4]}]

       ; Act
       (let [new-map (mask-na-cells map-matrix 6 na-values)]

         ; Assert
         (is (= new-map
                [nil nil nil nil nil nil
                 nil nil nil "5" "6" nil
                 nil nil nil "2" "3" nil
                 nil "3" nil nil nil nil
                 nil "4" nil nil nil nil
                 nil nil nil nil nil nil]))))))

(deftest mask-upper-border-na-cells
  (testing "Filter should mask upper border na cells and their neighbourhood."
    (
     ; Act
     let [map-matrix (mask-na-cells [1 "na" 1 1 1
                                     1 1 1 1 1
                                     1 1 1 1 1]
                                    5
                                    {0 [1]})]

       ; Assert
       (is (= map-matrix
              [nil nil nil nil nil
               nil nil nil 1 nil
               nil nil nil nil nil])))))

(deftest mask-lower-border-na-cells
  (testing "Filter should mask lower border na cells and their neighbourhood."
    (
     ; Act
     let [map-matrix (mask-na-cells [1 1 1 1 1
                                     1 1 1 1 1
                                     1 1 1 "na" 1]
                                    5
                                    {2 [3]})]

       ; Assert
       (is (= map-matrix
              [nil nil nil nil nil
               nil 1 nil nil nil
               nil nil nil nil nil])))))

(deftest mask-left-border-na-cells
  (testing "Filter should mask left border na cells and their neighbourhood."
    (
     ; Act
     let [map-matrix (mask-na-cells [1 1 1
                                     "na" 1 1
                                     1 1 1
                                     1 1 1
                                     1 1 1]
                                    3
                                    {1 [0]})]

       ; Assert
       (is (= map-matrix
              [nil nil nil
               nil nil nil
               nil nil nil
               nil 1 nil
               nil nil nil])))))

(deftest mask-right-border-na-cells
  (testing "Filter should mask right border na cells and their neighbourhood."
    (
     ; Act
     let [map-matrix (mask-na-cells [1 1 1
                                     1 1 1
                                     1 1 1
                                     1 1 "na"
                                     1 1 1]
                                    3
                                    {3 [2]})]

       ; Assert
       (is (= map-matrix
              [nil nil nil
               nil 1 nil
               nil nil nil
               nil nil nil
               nil nil nil])))))

(deftest mask-only-na-cells-test
  (testing "Filter should mask only na cells."
    (
     ; Act
     let [map-matrix (mask-only-na-cells [1 "na" 1 1 1
                                          "na" "na" 1 1 1
                                          1 1 1 "na" 1]
                                         5
                                         {0 [1] 1 [0 1] 2 [3]})]

       ; Assert
       (is (= map-matrix
              [1 nil 1 1 1
               nil nil 1 1 1
               1 1 1 nil 1])))))


;;; test grouping

(deftest mask-na-cells-test
  (testing "Na cells and border cells should be masked:\n"
    (mask-na-cells-and-their-neighbourhood)
    (mask-upper-border-na-cells)
    (mask-lower-border-na-cells)
    (mask-left-border-na-cells)
    (mask-right-border-na-cells)))


;;; tests in the namespace

(defn test-ns-hook
  "summary: Explicit definition of tests in the namespace.
   returns: Explicit definition of tests in the namespace."
  []
  (mask-na-cells-test)
  (mask-only-na-cells-test))
