;; !!!!!!!!!! from CalibrationEnergyABM project


(ns utilities.floating-point-comparison
  (:require [clojure.math.numeric-tower :as math]))


;;; private defs

;; Defines precision for operations with double numbers.
(def ^:private epsilon 0.00000001)

;;; private methods

(defn- relative-diff
  "Returns difference between the first double number
   and the second double number relative to their maximum.
      x: Double number value.
      y: Double number value.
   returns: Difference between two double numbers relative to their maximum."
  [x y]
  (/ (- x y) (max (math/abs x) (math/abs y))))


;;; public methods

(defn double=
  "Determines whether two double numbers (or vectors of double numbers) are equal.
      x: A vector of double numbers or single value.
      y: A vector of double numbers or single value.
   returns: True if the specified double numbers (or vectors of double numbers) are equal.
            Otherwise, false."
  [x y]
  (if (or (nil? x) (nil? y))
    false
    (if (and (number? x) (number? y))
      (recur (vector x) (vector y))
      (if (not (= (count x) (count y)))
        false
        (if (empty? x)
          true
          (let [number1 (first x) number2 (first y)]
            (if (not (if (<= (math/abs (- number1 number2)) epsilon)
                       true
                       (<= (math/abs (relative-diff number1 number2)) epsilon)))
              false
              (recur (rest x) (rest y)))))))))

(defn double<
  "Determines whether the first double argument is less than the second one.
      x: Double number value.
      y: Double number value.
   returns: True if the first double number is less than the second one.
            Otherwise, false."
  [x y]
  (if (double= x y)
    false
    (> (relative-diff y x) epsilon)))

(defn double>
  "Determines whether the first double argument is greater than the second one.
      x: Double number value.
      y: Double number value.
   returns: True if the first double number is greater than the second one.
            Otherwise, false."
  [x y]
  (double< y x))

(defn double<=
  "Determines whether the first double argument is less than or equal to the second one.
      x: Double number value.
      y: Double number value.
   returns: True if the first double number is less than or
            equal to the second one. Otherwise, false."
  [x y]
  (not (double> x y)))

(defn double>=
  "Determines whether the first double argument is greater than or equal to the second one.
      x: Double number value.
      y: Double number value.
   returns: True if the first double number is greater than or
            equal to the second one. Otherwise, false."
  [x y]
  (not (double< x y)))
