(ns utilities.format
  (:require [utilities.string-formatter :as string-formatter]))


;;; public methods

(defn double-to-str
  "Formats a string from the floating-point number.

   Arguments: value
              digits

   [value] is converted to string with the number of decimal places equal to [digits].

   Formatting is independent from locale with point used as decimal separator.

   Returns string in the floating-point format: [+-]([0-9]*[.])?[0-9]+

   ## Usage

       (require '[utilities.format :refer :all])

       (double-to-str 1.9 4)
       => \"1.9000\"

       (double-to-str -45.688 2)
       => \"-45.69\"

       (double-to-str +45.688 2)
       => \"45.69\"
  "
  [value digits]
  (-> (str "%."
           digits
           "f")
      (format value)
      string-formatter/to-numeric))
