(ns utilities.reader
  (:require [clojure.java.io :as io]
            [clojure.xml :as xml]))


;;; public methods

(defn read-file
  "Reads entire text file.

   Arguments: file-name

   File is read from [file-name].
   Returns text by lines.

   ## Usage

      (require '[utilities.reader :refer :all])

      (read-file \"map.asc\")
  "
  [file-name]
  (with-open [rdr (io/reader file-name)]
    (doall (line-seq rdr))))

(defn read-xml-file
  "Reads xml file.

   Arguments: path

   File is read from [file-name].
   Returns xml file content.

   ## Usage

      (require '[utilities.reader :refer :all])

      (read-xml-file \"settings.xml\")
  "
  [path]
  (xml/parse (io/file path)))
